@@ -1,59 +1,158 @@
 from __future__ import annotations
 
+import abc
 import getpass
 import logging
-from typing import Any, Iterable, Optional, Tuple, cast
-from urllib.parse import urlparse
+import os
+import shutil
+import subprocess
+from typing import Any, Callable, Iterable, Optional, Tuple, cast
+from urllib.parse import SplitResult, urlparse, urlsplit
 
 from requests import Response
 from requests.auth import AuthBase, HTTPBasicAuth
 from requests.models import PreparedRequest
 from requests.utils import get_netrc_auth
 
-from unearth.utils import split_auth_from_netloc, split_auth_from_url
+from unearth.utils import commonprefix, split_auth_from_url
 
-try:
-    import keyring  # type: ignore
-except ModuleNotFoundError:
-    keyring = None  # type: ignore[assignment]
+KEYRING_DISABLED = False
 
 AuthInfo = Tuple[str, str]
 MaybeAuth = Optional[Tuple[str, Optional[str]]]
 logger = logging.getLogger(__name__)
 
 
-def get_keyring_auth(url: str | None, username: str | None) -> AuthInfo | None:
-    """Return the tuple auth for a given url from keyring."""
-    global keyring
-    if not url or not keyring:
+def _expect_argument(func: Callable[..., Any], argname: str) -> bool:
+    """Return True if the function expects the argument."""
+    import inspect
+
+    sig = inspect.signature(func)
+    return argname in sig.parameters
+
+
+class KeyringBaseProvider(metaclass=abc.ABCMeta):
+    """Base class for keyring providers."""
+
+    @abc.abstractmethod
+    def get_auth_info(self, url: str, username: str | None) -> AuthInfo | None:
+        """Return the password for the given url and username.
+        The username can be None.
+        """
+        ...
+
+    @abc.abstractmethod
+    def save_auth_info(self, url: str, username: str, password: str) -> None:
+        """Set the password for the given url and username."""
+        ...
+
+
+class KeyringModuleProvider(KeyringBaseProvider):
+    """Keyring provider that uses the keyring module."""
+
+    def __init__(self) -> None:
+        import keyring  # type: ignore
+
+        self.keyring = keyring
+
+    def get_auth_info(self, url: str, username: str | None) -> AuthInfo | None:
+        if hasattr(self.keyring, "get_credential"):
+            logger.debug("Getting credentials from keyring for url: %s", url)
+            cred = self.keyring.get_credential(url, username)
+            if cred is not None:
+                return cred.username, cred.password
+
+        if username is None:
+            username = "__token__"
+        logger.debug("Getting password from keyring for: %s@%s", username, url)
+        password = self.keyring.get_password(url, username)
+        if password:
+            return username, password
+        return None
+
+    def save_auth_info(self, url: str, username: str, password: str) -> None:
+        self.keyring.set_password(url, username, password)
+
+
+class KeyringCliProvider(KeyringBaseProvider):
+    def __init__(self, cmd: str) -> None:
+        self.keyring = cmd
+
+    def get_auth_info(self, url: str, username: str | None) -> AuthInfo | None:
+        if username is None:
+            username = "__token__"
+        logger.debug("Getting password from keyring CLI for %s@%s", username, url)
+        password = self._get_password(url, username)
+        if password is not None:
+            return username, password
+        return None
+
+    def save_auth_info(self, url: str, username: str, password: str) -> None:
+        return self._set_password(url, username, password)
+
+    def _get_password(self, service_name: str, username: str) -> str | None:
+        """Mirror the implementation of keyring.get_password using cli"""
+        cmd = [self.keyring, "get", service_name, username]
+        env = dict(os.environ, PYTHONIOENCODING="utf-8")
+        res = subprocess.run(
+            cmd, stdin=subprocess.DEVNULL, capture_output=True, env=env
+        )
+        if res.returncode:
+            return None
+        return res.stdout.decode("utf-8").strip(os.linesep)
+
+    def _set_password(self, service_name: str, username: str, password: str) -> None:
+        """Mirror the implementation of keyring.set_password using cli"""
+        if self.keyring is None:
+            return None
+
+        cmd = [self.keyring, "set", service_name, username]
+        input_ = (password + os.linesep).encode("utf-8")
+        env = dict(os.environ, PYTHONIOENCODING="utf-8")
+        subprocess.run(cmd, input=input_, env=env, check=True)
+        return None
+
+
+def get_keyring_provider() -> KeyringBaseProvider | None:
+    """Return the keyring provider to use."""
+    if KEYRING_DISABLED:
         return None
 
     try:
-        try:
-            get_credential = keyring.get_credential
-        except AttributeError:
-            pass
-        else:
-            logger.debug("Getting credentials from keyring for %s", url)
-            cred = get_credential(url, username)
-            if cred is not None:
-                return cred.username, cred.password
-            return None
+        return KeyringModuleProvider()
+    except ImportError:
+        pass
+    except Exception as exc:
+        logger.warning(
+            "Importing keyring failed: %s, trying to find a keyring executable.",
+            exc,
+        )
 
-        if not username:
-            username = "__token__"
-        logger.debug("Getting password from keyring for %s@%s", username, url)
-        password = keyring.get_password(url, username)
-        if password:
-            return username, password
+    keyring = shutil.which("keyring")
+    if keyring is not None:
+        return KeyringCliProvider(keyring)
 
+    return None
+
+
+def get_keyring_auth(url: str | None, username: str | None) -> AuthInfo | None:
+    """Return the tuple auth for a given url from keyring."""
+    if not url:
+        return None
+
+    keyring = get_keyring_provider()
+    if keyring is None:
+        return None
+    try:
+        return keyring.get_auth_info(url, username)
     except Exception as exc:
         logger.warning(
             "Keyring is skipped due to an exception: %s",
             str(exc),
         )
-        keyring = None  # type: ignore[assignment]
-    return None
+        global KEYRING_DISABLED
+        KEYRING_DISABLED = True
+        return None
 
 
 class MultiDomainBasicAuth(AuthBase):
@@ -64,28 +163,42 @@ class MultiDomainBasicAuth(AuthBase):
         self._cached_passwords: dict[str, AuthInfo] = {}
         self._credentials_to_save: tuple[str, str, str] | None = None
 
-    def _get_auth_from_index_url(self, netloc: str) -> tuple[MaybeAuth, str | None]:
+    def _get_auth_from_index_url(self, url: str) -> tuple[MaybeAuth, str | None]:
         """Return the extracted auth and the original index URL matching
-        the requested netloc.
+        the requested url. The auth part should be already stripped from the URL.
 
         Returns None if no matching index was found, or if --no-index
         was specified by the user.
         """
-        if not netloc or not self.index_urls:
+        if not url or not self.index_urls:
             return None, None
 
-        for u in self.index_urls:
-            parsed = urlparse(u)
-            auth, index_netloc = split_auth_from_netloc(parsed.netloc)
-            if index_netloc == netloc:
-                return auth, u
-        return None, None
+        target = urlsplit(url.rstrip("/") + "/")
+        candidates: list[SplitResult] = []
+
+        for index in self.index_urls:
+            index = index.rstrip("/") + "/"
+            auth, url_no_auth = split_auth_from_url(index)
+            parsed = urlsplit(url_no_auth)
+            if parsed == target:
+                return auth, index
+
+            if target.netloc == parsed.netloc:
+                candidates.append(urlsplit(index))
+
+        if not candidates:
+            return None, None
+        best_match = max(
+            candidates, key=lambda x: commonprefix(x.path, target.path).rfind("/")
+        )
+        index = best_match.geturl()
+        return split_auth_from_url(index)[0], index
 
     def _get_new_credentials(
         self,
         original_url: str,
         *,
-        allow_netrc: bool = False,
+        allow_netrc: bool = True,
         allow_keyring: bool = False,
     ) -> tuple[str | None, str | None]:
         """Find and return credentials for the specified URL."""
@@ -102,7 +215,12 @@ class MultiDomainBasicAuth(AuthBase):
                 return cast(AuthInfo, auth)
 
         # Find a matching index url for this request
-        index_auth, index_url = self._get_auth_from_index_url(netloc)
+        # XXX: a compatiblity layer to support old function signature
+        if _expect_argument(self._get_auth_from_index_url, "netloc"):
+            index_auth, index_url = self._get_auth_from_index_url(netloc)
+        else:
+            index_auth, index_url = self._get_auth_from_index_url(url)
+
         if index_url:
             logger.debug("Found index url %s", index_url)
 
@@ -145,7 +263,9 @@ class MultiDomainBasicAuth(AuthBase):
         _, url = split_auth_from_url(original_url)
         netloc = urlparse(url).netloc
         # Try to get credentials from original url
-        username, password = self._get_new_credentials(original_url)
+        username, password = self._get_new_credentials(
+            original_url, allow_netrc=True, allow_keyring=False
+        )
 
         # If credentials not found, use any stored credentials for this netloc.
         # Do this if either the username or the password is missing.
@@ -190,32 +310,32 @@ class MultiDomainBasicAuth(AuthBase):
 
     # Factored out to allow for easy patching in tests
     def _should_save_password_to_keyring(self) -> bool:
-        if not keyring:
+        if get_keyring_provider() is None:
             return False
         return input("Save credentials to keyring [y/N]: ") == "y"
 
     def handle_401(self, resp: Response, **kwargs: Any) -> Response:
-        # We only care about 401 responses, anything else we want to just
+        # We only care about 401 response, anything else we want to just
         #   pass through the actual response
         if resp.status_code != 401:
             return resp
 
-        # We are not able to prompt the user so simply return the response
-        if not self.prompting:
-            return resp
-
         parsed = urlparse(cast(str, resp.url))
 
         # Query the keyring for credentials:
         username, password = self._get_new_credentials(
             resp.url,
-            allow_netrc=True,
+            allow_netrc=False,
             allow_keyring=True,
         )
 
         # Prompt the user for a new username and password
         save = False
         if not username and not password:
+            # We are not able to prompt the user so simply return the response
+            if not self.prompting:
+                return resp
+
             username, password, save = self._prompt_for_password(parsed.netloc)
 
         # Store the new username and password to use for future requests
@@ -254,21 +374,21 @@ class MultiDomainBasicAuth(AuthBase):
         """Response callback to warn about incorrect credentials."""
         if resp.status_code == 401:
             logger.warning(
-                "401 Error, Credentials not correct for %s",
+                "%s Error, Credentials not correct for %s",
+                resp.status_code,
                 resp.request.url,
             )
 
     def save_credentials(self, resp: Response, **kwargs: Any) -> None:
         """Response callback to save credentials on success."""
+        keyring = get_keyring_provider()
         assert keyring is not None, "should never reach here without keyring"
-        if not keyring:
-            return
 
         creds = self._credentials_to_save
         self._credentials_to_save = None
         if creds and resp.status_code < 400:
             try:
                 logger.info("Saving credentials to keyring")
-                keyring.set_password(*creds)
+                keyring.save_auth_info(*creds)
             except Exception:
                 logger.exception("Failed to save credentials")
