@@ -1,4 +1,5 @@
 """Configuration for the pytest test suite."""
+import os
 from ssl import SSLContext
 from unittest import mock
 
@@ -58,12 +59,12 @@ def fixtures_dir():
 def pypi():
     wsgi_app = create_app()
     with mock.patch.object(
-        PyPISession, "insecure_adapter_cls", return_value=WSGIAdapter(wsgi_app)
+        PyPISession, "insecure_adapter_cls", return_value=InsecureWSGIAdapter(wsgi_app)
     ):
         with mock.patch.object(
             PyPISession,
             "secure_adapter_cls",
-            return_value=InsecureWSGIAdapter(wsgi_app),
+            return_value=WSGIAdapter(wsgi_app),
         ):
             yield wsgi_app
 
@@ -71,7 +72,9 @@ def pypi():
 @pytest.fixture()
 def pypi_auth(pypi):
     def check_auth(auth):
-        return auth.username == "test" and auth.password == "password"
+        return auth.username == os.getenv(
+            "PYPI_USER", "test"
+        ) and auth.password == os.getenv("PYPI_PASSWORD", "password")
 
     def unauthenticated():
         message = {"message": "Unauthenticated"}
@@ -95,3 +98,9 @@ def session():
         yield s
     finally:
         s.close()
+
+
+@pytest.fixture(params=["html", "json"])
+def content_type(request, monkeypatch):
+    monkeypatch.setenv("INDEX_RETURN_TYPE", request.param)
+    return request.param
